<?php
// $Id: slot_machine.utils.inc,v 1.1.2.1 2008/12/05 22:58:01 mcarbone Exp $

/********************************************************************************
 * DISPLAY SLOTTED NODES
 ********************************************************************************/
function slot_machine_display_content($topic = NULL) {
  $vocab = variable_get('slot_machine_topics', 'none');
  if (is_numeric($vocab)) {
    if (!$topic && !variable_get('slot_machine_home_topic', 1)) {
      $topic_nids = array();
      $result = db_query('SELECT tid, name FROM {term_data} WHERE vid = %d', $vocab);
      while ($row = db_fetch_object($result)) {
        $topic_nid = db_result(db_query_range('SELECT sm.nid FROM {slot_machine} sm INNER JOIN {slot_machine_slots} s ON s.feature_type = sm.feature_type WHERE topic = %d ORDER BY s.weight ASC', $row->tid, 0, 1));
        if ($topic_nid) {
          $topic_nids[$row->name] = $topic_nid;
        }
      }
      if (!empty($topic_nids)) {
        return theme('slot_machine_all_topics', $topic_nids);
      }
    }
    else {
      $tid = db_result(db_query('SELECT tid FROM {term_data} WHERE name = "%s" AND vid = %d', $topic, $vocab));
      $feature_types = db_query('SELECT name FROM {slot_machine_slots} ORDER BY weight ASC');
      $nids = array();
      while ($row = db_fetch_object($feature_types)) {
        if (slot_machine_get_slot_nid($tid, $row->name)) {
          $nids[] = slot_machine_get_slot_nid($tid, $row->name);
        }
      }
      return theme('slot_machine_slot_nodes', $nids);
    }
  }
}

/********************************************************************************
 * FEATURED CONTENT QUEUE FUNCTIONS
 ********************************************************************************/

/**
  * Displays featured content queue page
  * 
  * @param int $topic - the tid of the topic to filter by; 0 when no filtering
  * @param string $type - string representing the feature type to filter by; 'all' when no filtering
  * @param int $limit - number of items per page
  * @param string $search - search terms for filtering
  * 
  * @return string - page output 
  */
function slot_machine_content_queue($topic = 0, $type = 'all', $limit = 20, $search = NULL) {
  // Get topics and pass them to javascript
  if (variable_get('slot_machine_home_topic', 1)) {
  $topics = array(0);
  } else {
    $topics = array();
  }
  $vocab = variable_get('slot_machine_topics', 'none');
  if ($vocab != 'none') {
    
    $get_topics = _sm_get_vocabulary_terms($vocab);
    foreach ($get_topics as $key => $mytopic) {
      $topics[] = $key;
    }
    
    drupal_add_js(array('topics' => $topics), 'setting', 'header', FALSE, TRUE, FALSE);
    drupal_add_js(drupal_get_path('module', 'slot_machine') .'/slot_machine.js', 'module', 'header', FALSE, TRUE, FALSE);
    drupal_add_css(drupal_get_path('module', 'slot_machine') .'/slot_machine.css');
    
    $table = drupal_get_form('slot_machine_content_queue_form', $topic, $type, $limit, $search);
    
    $output .= drupal_get_form('slot_machine_topic_type_form', $topic, $type, $limit, $search);
    $output .= theme('pager', NULL, $limit);
    $output .= $table;
    $output .= theme('pager', NULL, $limit);
    $output .= theme('slot_machine_per_page', $topic, $type, $limit, $search);
    
    return $output;
  }
  else {
    drupal_set_message(t('You have not chosen a vocubulary for your slot machines pages.'), 'error');
    return '';
  }
}

/*
 * Form for filtering by Topic or Feature Type
 *
 * @return array - a form array 
 */
function slot_machine_topic_type_form($form_state, $topic, $type, $limit, $search) {
  if (module_exists('search')) {
    $form['search'] = array(
      '#prefix' => '<div class="queue-search">',
      '#type' => 'textfield',
      '#size' => 15,
      '#title' =>'Search Content',
      '#default_value' => $search,
    );
    $form['go'] = array(
      '#suffix' => '</div>',
      '#type' => 'submit',
      '#value' => 'Go',
    );
  }
  
  // Get topics
  $topics = array(0 => 'All');
  $vocab = variable_get('slot_machine_topics', 'none');
  if ($vocab != 'none') {
    $topics += _sm_get_vocabulary_terms($vocab);
    
    // Get types
    $types = slot_machine_get_feature_types();
    
    $labels = theme('slot_machine_topic_type_labels', ($topic == 0 ? 'All Topics' : $topics[$topic]), ($type == 'all' ? 'All Features' : $types[$type]));
    $form['labels'] = array(
      '#type' => 'markup',
      '#value' => $labels,
    );
    
    $form['topic'] = array(
      '#type' => 'select',
      '#title' =>'Topic',
      '#options' => $topics,
      '#default_value' => $topic,
    );
    $form['type'] = array(
      '#type' => 'select',
      '#title' => 'Feature Type',
      '#options' => array_merge(array('all' => 'All'), $types),
      '#default_value' => $type,
    );
  
    $form['limit'] = array(
      '#type' => 'value',
      '#value' => ($limit ? $limit : 20),
    );
    
    $form['reset'] = array(
      '#type' => 'submit',
      '#value' => 'Reset',
      '#submit' => array('slot_machine_topic_type_form_reset'),
    );
    return $form;
  }
}

/*
 * Filter the content queue by the type/topic selected by the user.
 */
function slot_machine_topic_type_form_submit($form, &$form_state) {
  $form_state['redirect'] = 'admin/content/slotqueue/'. $form_state['values']['topic'] .'/'. $form_state['values']['type'] .'/'. $form_state['values']['limit'] . ($form_state['values']['search'] ? '/'. $form_state['values']['search'] : '');
}

function slot_machine_topic_type_form_reset($form, &$form_state) {
  $form_state['redirect'] = 'admin/content/slotqueue';
}

/**
  * The content queue table, created as a themed form so that we can have checkboxes embedded within
  */
function slot_machine_content_queue_form($form_state, $topic, $type, $limit, $search = NULL) {
  $vocab = variable_get('slot_machine_topics', 'none');
  if (is_numeric($vocab)) {
    $form['header'] = array(
      '#type' => 'value',
      '#value' => array(
        array('data' => theme('slot_machine_slot_all_header')),
        array('data' => theme('slot_machine_head_deck_header')),
        array('data' => theme('slot_machine_author_header')),
        array('data' => theme('slot_machine_topic_header')),
        array('data' => theme('slot_machine_feature_type_header')),
        array('data' => theme('slot_machine_published_header'), 'field' => 'n.created', 'sort' => 'desc'),
        array('data' => theme('slot_machine_audit_trail_header'))
      ),
    );

    $arguments = array(); // warning: the order in which this is filled is important

    // Topic filter
    $topic_join = $topic_where = '';
    $types = slot_machine_get_feature_types();
    $content_types = '"'. implode('","', array_keys($types)) .'"';

    if ($topic) {
      $q = 'SELECT t.nid, t.tid FROM {term_node} t JOIN {node} n ON n.nid = t.nid WHERE n.type IN ('. $content_types .')';
      db_query_temporary($q, 'temp_term_node');
      $topic_join = "JOIN {temp_term_node} tn ON tn.tid = %d";
      $topic_where = "AND tn.nid = n.nid";
      $arguments[] = $topic;
    }
  
    // Type filter
    $type_where = '';
    if ($type != 'all') {
      $type_where = "AND n.type = '%s'";
      $arguments[] = $type;
    }
  
    // Search filter, borrowing from search.module
    $search_join = $search_where = '';
    if (module_exists('search') && $search) {
      $query = search_parse_query($search);
      if ($query[2] == '') {
        drupal_set_message(t('Warning: You must search with at least one positive keyword with @count characters or more.', array('@count' => variable_get('minimum_word_size', 3))));
      }
      else {
        // First pass (OR search)
        $result = db_query_temporary("SELECT DISTINCT(i.sid) FROM {search_index} i WHERE (". $query[2] .")", $query[3], 'temp_search_sids');
  
        // Second pass (AND search), to be done in the table query
        $search_join = "JOIN temp_search_sids i ON i.sid = n.nid
                JOIN {search_dataset} d ON i.sid = d.sid";
        $search_where = "AND (". $query[0] .")";
        $arguments = array_merge($arguments, $query[1]);
      }
    }
    $q2 = 'SELECT n.nid, n.vid, n.type, n.created, n.status, n.uid, n.title FROM {node} n JOIN {node_revisions} nr ON n.vid = nr.vid WHERE n.type IN ('. $content_types .')';
    db_query_temporary($q2, 'temp_node');
    
    $sql = "SELECT n.nid, 
            n.title,
            n.uid,  
            n.type, 
            n.created
            FROM temp_node n 
              ". $search_join ."
              ". $topic_join ."
            WHERE n.status = 1
            ". $topic_where ."
            ". $type_where ."
            ". $search_where ." ";

    $sql .= ' '. tablesort_sql($form['header']['#value']);
    if ($limit) {
      $results = pager_query($sql, $limit, 0, NULL, $arguments);
    }
    else {
      $results = db_query($sql, $arguments);
    }

    while ($obj = db_fetch_object($results)) {
      // Grab topics that this node has been assigned
      $node_topics = array();
      $terms = db_query("SELECT td.name FROM {node} n JOIN {term_node} tn ON n.nid = tn.nid JOIN {term_data} td ON tn.tid = td.tid JOIN {vocabulary} v ON td.vid = v.vid WHERE v.vid = %d AND n.nid = %d", $vocab, ($obj->comment_nid ? $obj->comment_nid : $obj->nid));
      while ($term = db_fetch_object($terms)) {
        $node_topics[] = $term->name;
      }

      // Get all topics that this node has already been slotted with
      $slotted_topics = array();
      $slots = db_query("SELECT topic FROM {slot_machine} WHERE nid = %d", $obj->nid);
      while ($slot = db_fetch_object($slots)) {
        $slotted_topics[] = $slot->topic;
      }

      // Get topics for choose menu
      if (variable_get('slot_machine_home_topic', 1)) {
        $slot_topics = array('home_0' => 'Home');
      } else {
        $slot_topics = array();
      }
      $default_topics = array(in_array(0, $slotted_topics) ? 'home_0' : '');
      
      $my_topics = _sm_get_vocabulary_terms($vocab);

      foreach ($my_topics as $topic_tid => $topic_name) {
        $slot_topics[(in_array($topic_name, $node_topics) ? 'default_' : '') . $topic_tid] = $topic_name;
        $default_topics[] = (in_array($topic_tid, $slotted_topics) ? (in_array($topic_name, $node_topics) ? 'default_' : '') . $topic_tid : '');
      }

      $form['slot_actions'][$obj->nid] = array(
        '#type' => 'markup',
        '#value' => theme('slot_machine_slot_actions', $obj->nid),
      );
      $form['slot_topics']['cb'. $obj->nid] = array(
        '#type' => 'checkboxes',
        '#default_value' => $default_topics,
        '#options' => $slot_topics,
        '#prefix' => theme('slot_machine_popup_prefix', $obj->nid),
        '#suffix' => theme('slot_machine_popup_suffix', $obj->nid),
      );

      // Display head and deck
      $text = db_fetch_object(db_query("SELECT nr.title, 
              nr.body 
              FROM {node} n
                JOIN {node_revisions} nr ON n.vid = nr.vid
              WHERE n.nid = %d", $obj->nid));

      $head = $text->title;
      $deck = $text->body;

      $form['head_deck'][$obj->nid] = array(
        '#type' => 'markup', 
        '#value' => theme('slot_machine_head_deck', $head, $deck, $obj->nid, $obj->type),
      );
  
      if ($obj->uid) {
        $author = db_result(db_query("SELECt u.name FROM {users} u WHERE u.uid = %d", $obj->uid));
      }
      $form['author'][$obj->nid] = array(
        '#type' => 'markup', 
        '#value' => ($obj->uid == 0 ? ($obj->name ? $obj->name : 'Anonymous') : l($author, 'user/'. $obj->uid, array('attributes' => array('class' => 'author')))),
      );

      $form['topics'][$obj->nid] = array(
        '#type' => 'markup', 
        '#value' => implode(', ', $node_topics),
      );

      $form['feature_type'][$obj->nid] = array(
        '#type' => 'markup', 
        '#value' => $types[$obj->type],
      );

      $form['created'][$obj->nid] = array(
        '#type' => 'markup', 
        '#value' => format_date($obj->created, 'custom', 'n/j/y g:i a'),
      );

      // Grab audit trail
      $history = db_query("SELECT td.name, h.timestamp FROM {slot_machine_history} h LEFT OUTER JOIN {term_data} td ON h.topic = td.tid WHERE nid = %d ORDER BY timestamp DESC", $obj->nid);
      $trail = array();
      while ($item = db_fetch_object($history)) {
        $trail[] = format_date($item->timestamp, 'custom', 'n/j/y') .' '. ($item->name ? $item->name : 'Home');
      }
      $form['history'][$obj->nid] = array(
        '#type' => 'markup', 
        '#value' => theme('slot_machine_audit_trail', $trail),
      );
    }

    // Pass in the current URL with query arguments
    $query = '';
    if ($_GET['sort']) {
      $query .= 'sort='. $_GET['sort'];
    }
    if ($_GET['order']) {
      $query .= ($query ? '&' : '') . 'order='. $_GET['order'];
    }
    if ($_GET['page']) {
      $query .= ($query ? '&' : '') . 'page='. $_GET['page'];
    }
    $form['url'] = array(
      '#type' => 'value',
      '#value' => 'admin/content/slotqueue/'. $topic .'/'. $type .'/'. $limit .($search ? '/'. $search : ''),
    );
    $form['query'] = array(
      '#type' => 'value',
      '#value' => $query,
    );

    $form['reset'] = array(
      '#type' => 'submit',
      '#value' => 'Reset',
      '#submit' => array('slot_machine_content_queue_form_reset'),
    );
    $form['queue_submit'] = array(
      '#type' => 'submit',
      '#value' => 'Save',
    );
    return $form;
  }
}

/*
 * Slot the checked items.
 */
function slot_machine_content_queue_form_submit($form, &$form_state) {
  $items = 0;
  $slots = 0;
  foreach($form_state['values'] as $name => $item) {
    if (substr($name, 0, 2) == 'cb') {
      $nid = substr($name, 2);
      $slotted_item = false;
      foreach ($item as $key => $value) {
        if ($key && $value) {
          $slots++;
          $slotted_item = true;
          if ($value == 'home_0') {
            $topic = 0;
          }
          elseif (substr($value, 0, 7) == 'default') {
            $topic = substr($value, 8);
          }
          else {
            $topic = $value;
          }
          $type = slot_machine_get_feature_type($nid);
          
          // Slot the item at the end of the queue for that topic and type
          db_query("INSERT INTO {slot_machine} (nid, topic, feature_type, priority) SELECT %d, %d, '%s', IF((SELECT priority FROM {slot_machine} WHERE topic = %d AND feature_type = '%s' LIMIT 1) IS NULL, 0, (SELECT priority + 1 FROM {slot_machine} WHERE topic = %d AND feature_type = '%s' ORDER BY priority DESC LIMIT 1))", $nid, $topic, $type, $topic, $type, $topic, $type);
          
          // If this is slotted at the front of the queue, it's published, so we need to mark the audit trail
          $priority = db_result(db_query("SELECT m.priority FROM {slot_machine} m JOIN {slot_machine_slots} s ON m.priority = s.priority AND m.feature_type = s.feature_type WHERE m.nid = %d AND m.topic = %d AND m.feature_type = '%s'", $nid, $topic, $type));
          if (is_numeric($priority)) {
            db_query("UPDATE {slot_machine} SET schedule = UNIX_TIMESTAMP(now()) WHERE nid = %d AND topic = %d AND feature_type = '%s'", $nid, $topic, $type);
            db_query("INSERT INTO {slot_machine_history} (nid, topic, timestamp) VALUES (%d, %d, UNIX_TIMESTAMP(now()))", $nid, $topic);
          }
        }
      }
      if ($slotted_item) {
        $items++;
      }
    }
  }
  drupal_set_message(t('You have successfully slotted !items into !slots.', array('!items' => $items . format_plural($items, ' item', ' items'), '!slots' => $slots . format_plural($slots, ' slot', ' slots'))));
  
  if ($form_state['values']['query']) {
    return array(
      'href' => $form_state['values']['url'],
      'query' => $form_state['values']['query'],
    );
  }
}

function slot_machine_content_queue_form_reset($form, &$form_state) {
  $form_state['redirect'] = 'admin/content/slotqueue';
}

/********************************************************************************
 * SLOT MACHINE 2.0 FUNCTIONS
 ********************************************************************************/

/**
  * Displays Slot Machine 2.0 page
  * 
  * @param int $topic - the tid of the topic to filter by; 0 represents "Home"
  * @param string $type - string representing the feature type that is currently selected
  *
  * @return string - page output 
  */
function slot_machine_slot_machine($topic = 0, $type = NULL) {

  $vocab = variable_get('slot_machine_topics', 'none');
  if (is_numeric($vocab)) {

    slot_machine_insert_slot_frequencies();
    drupal_add_js(drupal_get_path('module', 'slot_machine') .'/slot_machine.js', 'module', 'header', FALSE, TRUE, FALSE);
    drupal_add_css(drupal_get_path('module', 'slot_machine') .'/slot_machine.css');
    
    // Update schedules
    slot_machine_update_slot_schedules();
  
    // Get topics
    if (variable_get('slot_machine_home_topic', 1)) {
      $topics = array(0 => 'Home');
    }
    else {
      $topics = array();
    }
    $topics += _sm_get_vocabulary_terms($vocab);
    
    // what is the default topic to show if home is not a topic?
    if ($topic == 0) {
      if (variable_get('slot_machine_home_topic', 1) == 0) {
        $tids = array_keys($topics);
        $topic = $tids[0];
      }
    }

    // Don't display this stuff when we're in Preview mode
    if ($_POST['op'] != 'Preview') {
      // The 'View Schedule for:' form and label
      $output = drupal_get_form('slot_machine_view_schedule_topic_form', $topic, $topics, $type);
      $output .= theme('slot_machine_view_schedule_topic_label', $topics[$topic]);
      $output .= t('Click on the slot name to show the relevant queue beneath the schedule.');

      $types = slot_machine_get_feature_types();
      if ($types[$type]) {
        // Queue header and table form
        $element2 = array(
          '#title' => theme('slot_machine_queue_header', $type),
          '#collapsible' => TRUE,
          '#collapsed' => FALSE,
          '#value' => drupal_get_form('slot_machine_queue_form', $topic, $type),
        ); 
      }
  
      // Schedule header and table form
      $element = array(
        '#title' => theme('slot_machine_schedule_header'),
        '#collapsible' => TRUE,
        '#collapsed' => FALSE,
        '#value' => drupal_get_form('slot_machine_schedule_form', $topic, $type),
      );

      $output .= theme('fieldset', $element);
      $output .= $element2 ? theme('fieldset', $element2) : '';
    }
    else {
      // Just the schedule header and table form (preview)
      $element = array(
        '#title' => theme('slot_machine_schedule_header', $topics[$topic]),
        '#collapsible' => FALSE,
        '#collapsed' => FALSE,
        '#value' => drupal_get_form('slot_machine_schedule_form', $topic, $type),
      );
      $output .= theme('fieldset', $element);
    } 

  } else {
    drupal_set_message(t('You have not chosen a vocubulary for your slot machines pages'), 'error');
    $output = '';
  }
  return $output;  
}

/*
 * Form for selecting which schedule to display, by topic.
 */
function slot_machine_view_schedule_topic_form($form_state, $topic, $topics) {
  $form['topic'] = array(
    '#type' => 'select',
    '#title' => 'View Schedule for',
    '#default_value' => $topic,
    '#options' => $topics,
    '#attributes' => array('current_topic' => $topic),
  );
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => 'Submit',
    '#attributes' => array('style' => 'display:none'),
  );
  return $form;
}

/*
 * Refresh the slot machine page with the selected schedule topic
 */
function slot_machine_view_schedule_topic_form_submit($form, &$form_state) {
  $form_state['redirect'] = 'admin/content/slotmachine/'. $form_state['values']['topic'];
}

/**
  * Make sure there's a slot frequency for each slot/topic combination
  * Will also change frequencies for content rotation slots
  */
function slot_machine_insert_slot_frequencies() {
  $vocab = variable_get('slot_machine_topics', 'none');
  $rot_enabled_types = variable_get('slot_machine_enable_content_rotation', array());
  
  // Get topics
  $topics = array(0 => 0);
  if (is_numeric($vocab)) {
    $my_topics = _sm_get_vocabulary_terms($vocab);
    foreach ($my_topics as $topic_tid => $topic_name) {
      $topics[] = $topic_tid;
    }

    $sids = array();
    $feature_types = db_query('SELECT sid, feature_type FROM {slot_machine_slots}');
    while ($row = db_fetch_object($feature_types)) {
      $sids[] = $row->sid;
      foreach ($topics as $topic) {
        $rotbase_freq = NULL;
        if ($rot_enabled_types[$row->feature_type]) {
          $rotbase_freq = db_result(db_query('SELECT f.fid FROM {slot_machine_frequencies} f JOIN {slot_machine_slots_frequencies} sf ON f.fid = sf.fid JOIN {slot_machine_slots} s ON sf.sid = s.sid WHERE sf.topic = %d AND s.priority = 0 AND s.feature_type = "%s"', $topic, $row->feature_type));
        }
        $frequency = db_result(db_query('SELECT fid FROM {slot_machine_slots_frequencies} WHERE sid = %d AND topic = %d', $row->sid, $topic));
        if (!$frequency) {
          db_query('INSERT INTO {slot_machine_slots_frequencies} (sid, topic, fid) VALUES (%d, %d, %d)', $row->sid, $topic, $rotbase_freq ? $rotbase_freq : 1);
        }
        else if ($rotbase_freq > 1) {
          db_query('UPDATE {slot_machine_slots_frequencies} SET fid = %d WHERE sid = %d AND topic = %d', $rotbase_freq, $row->sid, $topic);
        }
      }
    }	
  }
}

/**
  * The schedule table, created as a themed form so that we can have selects embedded within
  */
function slot_machine_schedule_form($form_state, $topic, $type) {
  // Get what step we're on.  There are only two steps; step 2 is the preview page.
  $step = isset($form_state['values']) ? (int) $form_state['storage']['step'] : 0;

  // Store next step
  $form_state['storage']['step'] = $step + 1;

  // Store freqency options
  $frequencies = array();
  $results = db_query("SELECT fid, name FROM {slot_machine_frequencies}");
  while ($freq = db_fetch_object($results)) {
    $frequencies[$freq->fid] = $freq->name;
  }

  // Make rotation information available to javascript
  $rotations = array();
  $rotatable_types_array = slot_machine_get_rotatable_types();
  if (count($rotatable_types_array)) {
    $rotatable_types = array_keys($rotatable_types_array);
    $rotations = array();
    foreach ($rotatable_types_array as $type_array) {
      foreach ($type_array as $k => $ksid) {
        foreach ($type_array as $j => $jsid) {
          if ($j > $k) {
            $rotations[$ksid][] = $jsid; 
          }
        }
      }
    }
  }
  else {
    $rotatable_types = array();
  }
  drupal_add_js(array('rotations' => $rotations), 'setting', 'header', FALSE, TRUE, FALSE);

  $rot_enabled_types = variable_get('slot_machine_enable_content_rotation', array());
  if ($step == 1) {
    $form['header'] = array(
      '#type' => 'value',
      '#value' => array(
        array('data' => t('Slot')),
        array('data' => t('Remove')),
        array('data' => theme('slot_machine_head_deck_header')),
        array('data' => theme('slot_machine_author_header')),
        array('data' => theme('slot_machine_topic_header')),
        array('data' => theme('slot_machine_published_header')),
        array('data' => theme('slot_machine_recent_activity_header')),
        array('data' => theme('slot_machine_live_schedule_header')),
      ),
    );
    
    $results = db_query("SELECT sf.sid, sf.fid, s.name, s.feature_type, s.priority FROM {slot_machine_slots_frequencies} sf JOIN {slot_machine_slots} s ON sf.sid = s.sid JOIN {slot_machine_frequencies} f ON sf.fid = f.fid WHERE sf.topic = %d ORDER BY s.weight ASC, s.priority ASC", $topic);
    
    while ($obj = db_fetch_object($results)) { 
      // if we're on a rotatable type, disable frequency select for lower priority slots
      if (!empty($rotatable_types_array)) {
        $disable = FALSE;       
        if (in_array($obj->feature_type, $rotatable_types)) {
          if ($obj->fid != 1) {
            $rotate = TRUE;
          }
          $type_sids = $rotatable_types_array[$obj->feature_type];
          foreach ($type_sids as $k => $type_sid) {
            if ($obj->sid == $type_sid && $k > 0) {
              $rotbase_freq = db_result(db_query('SELECT f.name FROM {slot_machine_frequencies} f JOIN {slot_machine_slots_frequencies} sf ON f.fid = sf.fid JOIN {slot_machine_slots} s ON sf.sid = s.sid WHERE sf.topic = %d AND s.sid = %d', $topic, $type_sids[0]));
              if ($rotbase_freq != 'Never') {
                $disable = TRUE;
              }
              else {
                $rotate = FALSE;
              }
            }
          }        
        }
      }

      // Display the select dropdown for changing the slot frequency
      $rdest = NULL;
      if ($rot_enabled_types[$obj->feature_type] && $rotate) {
        $rdest = db_result(db_query("SELECT name FROM {slot_machine_slots} WHERE feature_type = '%s' AND priority = %d", $obj->feature_type, $obj->priority + 1));
      }
      $form['slot_label'][$obj->sid] = array(
        '#type' => 'markup', 
        '#value' => theme('slot_machine_slot_label', $topic, $obj->feature_type, $obj->name, ($obj->feature_type != $type), $rdest),
      );
      
      $form['slot_frequencies'][$obj->sid]['slot_frequencies_'. $obj->sid] = array(
        '#type' => 'select',
        '#default_value' => $obj->fid,
        '#options' => $frequencies,
        '#disabled' => $disable,
        '#attributes' => array('original' => $obj->fid),
      );
    
      // Now let's grab the node that is currently published in this slot
      $n = NULL;
      $nid = db_result(db_query("SELECT nid FROM {slot_machine} WHERE topic = %d AND feature_type = '%s' AND priority = %d", $topic, $obj->feature_type, $obj->priority));
      
      // Display remove checkbox and rotate checkbox if applicable
      if ($nid) {
        $n = node_load($nid);
        $form['remove'][$obj->sid]['remove_'. $obj->sid] = array(
          '#type' => 'checkbox',
          '#attributes' => array('class' => 'remove'),
          '#return_value' => $nid,
        );
        
        if ($rotate) {
          foreach ($type_sids as $k => $type_sid) {
            if ($obj->sid == $type_sid && $k < count($type_sids)-1) {
              $form['rot'][$obj->sid]['rot_'. $obj->sid] = array(
                '#type' => 'checkbox',
                '#title' => t('rotate'),
                '#attributes' => array('class' => 'rotate'),
                '#prefix' => theme('slot_machine_rot_checkbox_prefix', $obj->sid),
                '#suffix' => theme('slot_machine_rot_checkbox_suffix'),
              );
            } 
          }
        }
      }
      else {
        $form['remove'][$obj->sid] = array(
          '#type' => 'markup', 
          '#value' => '',
        );
      }
    
      // Display head and deck
      $head = $deck = '';
      if ($nid) {

        $head = $n->title;
        $deck = $n->body;

      }
      $form['head_deck'][$obj->sid] = array(
        '#type' => 'markup', 
        '#value' => ($nid ? theme('slot_machine_head_deck', $head, $deck, $nid, $obj->feature_type) : ''),
      );
    
      $author = _sm_get_author($nid);

      $form['author'][$obj->sid] = array(
        '#type' => 'markup', 
        '#value' => $author,
      );
      // Grab topics that this node has been assigned
      $node_topics = _sm_get_nodetopics($nid);

      $form['topics'][$obj->sid] = array(
        '#type' => 'markup', 
        '#value' => ($nid ? implode(', ', $node_topics) : ''),
      );

      $form['created'][$obj->sid] = array(
        '#type' => 'markup', 
        '#value' => ($nid ? format_date($n->created, 'custom', 'n/j/y g:i a') : ''),
      );

      // Display where this item's recent activity
      $max_priority = db_result(db_query("SELECT priority FROM {slot_machine_slots} WHERE feature_type = '%s' ORDER BY priority DESC", $obj->feature_type));
      if ($nid) {
        $trail = array();
        
        // Where the item has appeared in the last seven days
        $schedule = db_query("SELECT td.name, td.tid, h.timestamp FROM {slot_machine_history} h LEFT OUTER JOIN {term_data} td ON h.topic = td.tid WHERE nid = %d AND h.timestamp > %d ORDER BY h.timestamp ASC", $nid, time() - (7 * 24 * 60 * 60));
        while ($item = db_fetch_object($schedule)) {
          $trail[] = theme('slot_machine_recent_activity_item', ($item->name ? $item->name : 'Home'), format_date($item->timestamp, 'custom', 'n/j/y'), slot_machine_check_rot($nid, $obj->feature_type, ($item->tid ? $item->tid : 0)));
        }
        
        // Where the item is currently scheduled to appear
        $schedule = db_query("SELECT td.name, td.tid, s.schedule FROM {slot_machine} s LEFT OUTER JOIN {term_data} td ON s.topic = td.tid WHERE nid = %d AND schedule IS NOT NULL AND s.priority > %d ORDER BY schedule ASC", $nid, $max_priority);
        while ($item = db_fetch_object($schedule)) {
          $trail[] = theme('slot_machine_recent_activity_item', ($item->name ? $item->name : 'Home'), format_date($item->schedule, 'custom', 'n/j/y'), slot_machine_check_rot($nid, $obj->feature_type, ($item->tid ? $item->tid : 0)), TRUE);
        }
      }
      $form['schedule'][$obj->sid] = array(
        '#type' => 'markup', 
        '#value' => ($nid ? theme('slot_machine_audit_trail', $trail) : ''),
      );

      $live = db_result(db_query("SELECT schedule FROM {slot_machine} WHERE topic = %d AND feature_type = '%s' AND priority = %d", $topic, $obj->feature_type, $obj->priority));
      $form['live'][$obj->sid] = array(
        '#type' => 'markup', 
        '#value' => ($nid ? format_date($live, 'custom', 'n/j/y g:i a') : ''),
      );
    }
    
    $form['reset'] = array(
      '#type' => 'button',
      '#value' => t('Reset'),
    );  
    $form['preview'] = array(
      '#type' => 'submit',
      '#value' => t('Preview'),
    );
  }
  elseif ($step == 2) {
    $messages = array();
    $rotating = array(); // per-type, stores any rotating nodes
    $errors = array();

    // If the highest priority content rotation slot is set to something other than 'Never', we
    // set lower priority slots to have the same frequency
    if (!empty($rotatable_types_array)) {
      foreach ($rotatable_types_array as $ftype => $slots) {
        $fid1 = db_result(db_query("SELECT fid FROM {slot_machine_slots_frequencies} WHERE sid = %d AND topic = %d", $slots[0], $topic));
        for ($i = 0; $i < count($slots); $i++) {
          $orig_fid[$slots[$i+1]] = db_result(db_query("SELECT fid FROM {slot_machine_slots_frequencies} WHERE sid = %d AND topic = %d", $slots[$i+1], $topic));
          if ($form_state['values']['slot_frequencies_'. $slots[0]] != 1 && $form_state['values']['slot_frequencies_'. $slots[0]] != $fid1) {
            $form_state['values']['slot_frequencies_'. $slots[$i+1]] = $form_state['values']['slot_frequencies_'. $slots[0]];
          }
        }
      }
    }
    // Display the preview table
    $header = array(
      array('data' => t('Slot')),
      array('data' => theme('slot_machine_head_deck_header')),
      array('data' => theme('slot_machine_author_header')),
      array('data' => theme('slot_machine_topic_header')),
      array('data' => theme('slot_machine_published_header')),
      array('data' => theme('slot_machine_recent_activity_header')),
    );

    $results = db_query("SELECT sf.sid, s.name, s.feature_type, s.priority, s.weight FROM {slot_machine_slots_frequencies} sf JOIN {slot_machine_slots} s ON sf.sid = s.sid JOIN {slot_machine_frequencies} f ON sf.fid = f.fid WHERE sf.topic = %d ORDER BY s.priority ASC", $topic);

    while ($obj = db_fetch_object($results)) {
      $row = array();
      
      // Display the new slot frequency
      $row[] = theme('slot_machine_preview_label', $obj->name, ($form_state['values']['slot_frequencies_'. $obj->sid] ? $frequencies[$form_state['values']['slot_frequencies_'. $obj->sid]] : $frequencies[$orig_fid[$obj->sid]]));
      
      // Now let's grab the node that will be published in this slot if this form is submitted
      $n = $next_nid = NULL;
      
      $nid = db_result(db_query("SELECT nid FROM {slot_machine} WHERE topic = %d AND feature_type = '%s' AND priority = %d", $topic, $obj->feature_type, $obj->priority));
      $max_priority = db_result(db_query("SELECT priority FROM {slot_machine_slots} WHERE feature_type = '%s' ORDER BY priority DESC", $obj->feature_type));

      if ($form_state['values']['remove_'. $obj->sid]) {
        if ($nid == $form_state['values']['remove_'. $obj->sid]) {
        
          // If there is a node waiting to be rotated, put it in
          if ($rotating[$obj->feature_type]) {
            $old_nid = $nid;
            $nid = $next_nid = $rotating[$obj->feature_type];
            if ($form_state['values']['rot_'. $obj->sid]) {
              $rotating[$obj->feature_type] = $old_nid;
            }
            else {
              $rotating[$obj->feature_type] = NULL;
            }
          }
          // Else find the priority of the item that's next in the queue
          else {
            if ($obj->priority == 0) {
              $next_priority = $max_priority + 1;
            }
            else {
              $removed_sids = 0;
              $sids = db_query("SELECT sid FROM {slot_machine_slots} WHERE feature_type = '%s' AND priority < %d", $obj->feature_type, $obj->priority);
              while ($sid = db_fetch_object($sids)) {
                if ($form_state['values']['remove_'. $sid->sid] && !$errors[$sid->sid]) {
                  $removed_sids++;
                }
              }
              // this computation is dependent on the fact that if something is taking from the
              // queue, there has been no rotation of this type so far -- I believe this is 
              // enforced by the jQuery control of the rotate boxes
              $next_priority = $max_priority + $removed_sids + 1;
            }
          
            $next_nid = db_result(db_query("SELECT nid FROM {slot_machine} WHERE topic = %d AND feature_type = '%s' AND priority = %d", $topic, $obj->feature_type, $next_priority));
            if ($next_nid) {
              if ($form_state['values']['rot_'. $obj->sid]) {
                $rotating[$obj->feature_type] = $nid;
              }
              $nid = $next_nid;
            }
            else {
              $messages[] = t('The item in the @slot slot will not be removed because the queue is empty.', array('@slot' => $obj->name));
            }
          }
        }
        else {
          if ($form_state['values']['rot_'. $obj->sid]) {
            $messages[] = t('The item in the @slot slot has already been removed, either by another editor or by cron.  As such, the content in this slot will not be rotated.  Press \'Cancel\' to see the current schedule.', array('@slot' => $obj->name));
          }
          else {
            $messages[] = t('The item in the @slot slot has already been removed, either by another editor or by cron. Press \'Cancel\' to see the current schedule.', array('@slot' => $obj->name));
          }
          $errors[$obj->sid] = 1;
        }
      }
      
      // Display head and deck
      $head = $deck = '';
      if ($nid) {
        $n = node_load($nid);
        $head = $n->title;
        $deck = $n->body;
      }
      $row[] = ($nid ? theme('slot_machine_head_deck', $head, $deck, $nid, $obj->feature_type) : '');
  
      // Display author
      $author = _sm_get_author($nid);

      $row[] = $author;
    
      // Grab topics that this node has been assigned
      $node_topics = _sm_get_nodetopics($nid);
      $row[] = ($nid ? implode(', ', $node_topics) : '');

      $row[] = ($nid ? format_date($n->created, 'custom', 'n/j/y g:i a') : '');

      // Display where this item is scheduled
      if ($nid) {
        $trail = array();
        
        // Where the item has appeared in the last seven days
        $schedule = db_query("SELECT td.name, td.tid, h.timestamp FROM {slot_machine_history} h LEFT OUTER JOIN {term_data} td ON h.topic = td.tid WHERE nid = %d AND h.timestamp > %d ORDER BY h.timestamp ASC", $nid, time() - (7 * 24 * 60 * 60));
        while ($item = db_fetch_object($schedule)) {
          $trail[] = theme('slot_machine_recent_activity_item', ($item->name ? $item->name : 'Home'), format_date($item->timestamp, 'custom', 'n/j/y'), slot_machine_check_rot($nid, $obj->feature_type, ($item->tid ? $item->tid : 0)));
        }
        
        // Where the item is currently scheduled to appear
        $schedule = db_query("SELECT td.name, td.tid, s.schedule FROM {slot_machine} s LEFT OUTER JOIN {term_data} td ON s.topic = td.tid WHERE nid = %d AND schedule IS NOT NULL AND s.priority > %d ORDER BY schedule ASC", $nid, $max_priority);
        while ($item = db_fetch_object($schedule)) {
          $trail[] = theme('slot_machine_recent_activity_item', ($item->name ? $item->name : 'Home'), format_date($item->schedule, 'custom', 'n/j/y'), slot_machine_check_rot($nid, $obj->feature_type, ($item->tid ? $item->tid : 0)), TRUE);
        }
      }
      $row[] =  ($nid ? theme('slot_machine_audit_trail', $trail) : '');
      
      $rows[] = array(
        'data' => $row,
        'class' => ($next_nid ? 'selected' : ''),
        'weight' => $obj->weight,
        'priority' => $obj->priority,
      );
    }

    // resort the rows by weight
    usort($rows, '_slot_machine_cmp_preview_rows');

    $form['preview_schedule'] = array(
      '#type' => 'markup',
      '#value' => theme('table', $header, $rows),
    );
    
    if ($_POST['op'] == 'Preview') {
      foreach ($messages as $message) {
        drupal_set_message($message);
      }
    }
    
    // Put form values from step 1 in storage
    $slot_ids = db_query("SELECT sid FROM {slot_machine_slots} ORDER BY sid");
    while ($slot = db_fetch_object($slot_ids)) {
      $form_state['storage']['slot_frequencies_'. $slot->sid] = $form_state['values']['slot_frequencies_'. $slot->sid];
      $form_state['storage']['remove_'. $slot->sid] = $form_state['values']['remove_'. $slot->sid];
      $form_state['storage']['rot_'. $slot->sid] = $form_state['values']['rot_'. $slot->sid];
    }
    
    $form['topic'] = array(
      '#type' => 'value',
      '#value' => $topic,
    );
    $form['type'] = array(
      '#type' => 'value',
      '#value' => $type,
    );
    
    $form['cancel'] = array(
      '#type' => 'submit',
      '#value' => t('Cancel'),
      '#submit' => array('slot_machine_schedule_form_cancel'),
    );  
    $form['preview_submit'] = array(
      '#type' => 'submit',
      '#value' => t('Save'),
    );
  }
  return $form;
}

function _slot_machine_cmp_preview_rows($a, $b) {
  if ($a['weight'] == $b['weight']) {
    if ($a['priority'] == $b['priority']) {
      return 0;
    }
    return ($a['priority'] > $b['priority']) ? 1 : -1; 
  }
  return ($a['weight'] > $b['weight']) ? 1 : -1;
}

/*
 * Update frequencies and remove and replace items in the schedule.
 */
function slot_machine_schedule_form_submit($form, &$form_state) {
  if ($form_state['storage']['step'] < 3) {
    return;
  }
  else {
    $topic = $form_state['values']['topic'];
    $rotating = array(); // per-type, stores any rotating nodes
      
    $results = db_query("SELECT sf.sid, s.name, s.feature_type, s.priority FROM {slot_machine_slots_frequencies} sf JOIN {slot_machine_slots} s ON sf.sid = s.sid JOIN {slot_machine_frequencies} f ON sf.fid = f.fid WHERE sf.topic = %d ORDER BY s.priority ASC", $topic);
    while ($obj = db_fetch_object($results)) {
      // Update the slot frequencies
      if ($form_state['storage']['slot_frequencies_'. $obj->sid]) {
        db_query("UPDATE {slot_machine_slots_frequencies} SET fid = %d WHERE sid = %d AND topic = %d", $form_state['storage']['slot_frequencies_'. $obj->sid], $obj->sid, $topic);
      }
        
      // Remove item and move up item from the top of the queue, if still applicable
      if ($form_state['storage']['remove_'. $obj->sid]) {
        $nid = db_result(db_query("SELECT nid FROM {slot_machine} WHERE topic = %d AND feature_type = '%s' AND priority = %d", $topic, $obj->feature_type, $obj->priority));
        $max_priority = db_result(db_query("SELECT priority FROM {slot_machine_slots} WHERE feature_type = '%s' ORDER BY priority DESC", $obj->feature_type));
        if ($nid == $form_state['storage']['remove_'. $obj->sid]) {

          // If there is a node waiting to be rotated, put it in
          if ($rotating[$obj->feature_type]) {
            db_query("UPDATE {slot_machine} SET nid = %d, schedule = UNIX_TIMESTAMP(now()) WHERE priority = %d AND topic = %d AND feature_type = '%s'", $rotating[$obj->feature_type], $obj->priority, $topic, $obj->feature_type);
            db_query("INSERT INTO {slot_machine_history} (nid, topic, timestamp) SELECT nid, topic, schedule FROM {slot_machine} WHERE priority = %d AND topic = %d AND feature_type = '%s'", $obj->priority, $topic, $obj->feature_type);
            drupal_set_message(t('The item in the @slot slot has been replaced with rotated content.', array('@slot' => $obj->name)));
            if ($form_state['storage']['rot_'. $obj->sid]) {
              $rotating[$obj->feature_type] = $nid;
            }
            else {
              $rotating[$obj->feature_type] = NULL;
            }
          }
          else {
            $num_queue = db_result(db_query("SELECT COUNT(nid) FROM {slot_machine} WHERE topic = %d AND feature_type = '%s' AND priority > %d", $topic, $obj->feature_type, $max_priority));
            if ($num_queue) {
              if ($form_state['storage']['rot_'. $obj->sid]) {
                $rotating[$obj->feature_type] = $nid;
              }
              // Remove the item, and update the others
              db_query("DELETE FROM {slot_machine} WHERE nid = %d AND topic = %d", $nid,  $topic);
              db_query("UPDATE {slot_machine} SET priority = %d, schedule = UNIX_TIMESTAMP(now()) WHERE priority = %d AND topic = %d AND feature_type = '%s'", $obj->priority, ($max_priority + 1), $topic, $obj->feature_type);
              db_query("INSERT INTO {slot_machine_history} (nid, topic, timestamp) SELECT nid, topic, schedule FROM {slot_machine} WHERE priority = %d AND topic = %d AND feature_type = '%s'", $obj->priority, $topic, $obj->feature_type);
              db_query("UPDATE {slot_machine} SET priority = priority - 1 WHERE priority > %d AND topic = %d AND feature_type = '%s' ORDER BY priority ASC", ($max_priority + 1), $topic, $obj->feature_type);            
              drupal_set_message(t('The item in the @slot slot has been removed.', array('@slot' => $obj->name)));
            }
            else {
              drupal_set_message(t('The item in the @slot slot has not been removed because the queue is empty.', array('@slot' => $obj->name)));
            }
          }
        }
        else {
          drupal_set_message(t('The item in the @slot slot has already been removed, either by another editor or by cron.', array('@slot' => $obj->name)));
        }
      }
    }
    unset($form_state['storage']);
    drupal_set_message(t('Any changes have been saved.'));
    $form_state['redirect'] = 'admin/content/slotmachine/'. $topic .'/'. $form_state['values']['type'];
  }
}

function slot_machine_schedule_form_cancel($form, &$form_state) {
  unset($form_state['storage']);
  drupal_set_message(t('Your changes have been cancelled.'));
  $form_state['redirect'] = 'admin/content/slotmachine/'. $form_state['values']['topic'] .'/'. $form_state['values']['type'];
}

/*
 * Returns true if the node is slotted and scheduled to eventually be automatically rotated
 */
function slot_machine_check_rot($nid, $type, $topic) {
  $rotatable_types_array = slot_machine_get_rotatable_types();  
  if (count($rotatable_types_array)) {
    $rotatable_types = array_keys($rotatable_types_array);
    if (in_array($type, $rotatable_types)) {
      $rotbase_freq = db_result(db_query("SELECT f.name FROM {slot_machine_frequencies} f JOIN {slot_machine_slots_frequencies} sf ON f.fid = sf.fid JOIN {slot_machine_slots} s ON sf.sid = s.sid WHERE s.feature_type = '%s' AND s.priority = 0 AND sf.topic = %d", $type, $topic));
      if ($rotbase_freq == 'Never') {
        return false;
      }
      else {
        $max_priority = db_result(db_query("SELECT priority FROM {slot_machine_slots} WHERE feature_type = '%s' ORDER BY priority DESC", $type));
        $is_slotted = db_result(db_query("SELECT nid FROM {slot_machine} WHERE priority < %d AND nid = %d AND topic = %d", $max_priority, $nid, $topic));
        if ($is_slotted) {
          return true;
        }
      }
    }
  }
  return false;
}

/**
  * The queue table, created as a themed form so that we can have priority text boxes embedded within
  */
function slot_machine_queue_form($form_state, $topic, $type) {
  $form['header'] = array(
    '#type' => 'value',
    '#value' => array(
      array('data' => theme('slot_machine_priority_header'), 'field' => 'sm.priority', 'sort' => 'asc'),
      array('data' => theme('slot_machine_select_all_header')),
      array('data' => theme('slot_machine_head_deck_header'), 'field' => 'text'),
      array('data' => theme('slot_machine_author_header'), 'field' => 'author'),
      array('data' => theme('slot_machine_topic_header')),
      array('data' => theme('slot_machine_published_header'), 'field' => 'n.created'),
      array('data' => theme('slot_machine_recent_activity_header')),
      array('data' => theme('slot_machine_scheduled_header'), 'field' => 'sm.schedule'),
    ),
  );

  $max_priority = db_result(db_query("SELECT priority FROM {slot_machine_slots} WHERE feature_type = '%s' ORDER BY priority DESC", $type));
        
  // This monster query lets us use Drupal's tablesort_sql for sorting columns
  $sql = "SELECT DISTINCT(n.nid), 
                 nr.title, 
                 nr.body, 
                 CASE n.uid WHEN 0 THEN 'Anonymous' ELSE u.name END as author, 
                 n.uid,  
                 n.type, 
                 n.created,
                 sm.priority,
                 sm.schedule
               FROM {node} n 
                 JOIN {node_revisions} nr ON n.vid = nr.vid
                 JOIN {slot_machine} sm ON n.nid = sm.nid
                 JOIN {users} u ON n.uid = u.uid
               WHERE sm.topic = %d 
                 AND sm.feature_type = '%s'
                 AND sm.priority > %d";
                 
  $sql .= ' '. tablesort_sql($form['header']['#value']);
  $results = db_query($sql, $topic, $type, $max_priority);
  
  while ($obj = db_fetch_object($results)) {
    $form['priority'][$obj->nid]['priority_'. $obj->nid] = array(
      '#type' => 'textfield',
      '#default_value' => $obj->priority - $max_priority,
      '#size' => 2,
      '#maxlength' => 3,
      '#attributes' => array('original' => $obj->priority - $max_priority),
    );
    $form['original_priority'][$obj->nid]['original_priority_'. $obj->nid] = array(
      '#type' => 'value',
      '#value' => $obj->priority - $max_priority,
    );

    $form['remove'][$obj->nid]['remove_'. $obj->nid] = array(
      '#type' => 'checkbox',
      '#return_value' => 1,
    );

    // Display head and deck
    $head = $obj->title;
    $deck = $obj->body;

    $form['head_deck'][$obj->nid] = array(
      '#type' => 'markup', 
      '#value' => theme('slot_machine_head_deck', $head, $deck, $obj->nid, $obj->type),
    );

    $form['author'][$obj->nid] = array(
      '#type' => 'markup', 
      '#value' => ($obj->uid == 0 ? $obj->author : l($obj->author, 'user/'. $obj->uid)),
    );

    // Grab topics that this node has been assigned
    $node_topics = array();
    $vocab = variable_get('slot_machine_topics', 'none');
    if (is_numeric($vocab)) {
      $terms = db_query("SELECT td.name FROM {node} n JOIN {term_node} tn ON n.nid = tn.nid JOIN {term_data} td ON tn.tid = td.tid JOIN {vocabulary} v ON td.vid = v.vid WHERE v.vid = %d AND n.nid = %d", $vocab, $obj->nid);
      while ($term = db_fetch_object($terms)) {
        $node_topics[] = $term->name;
      }
    }
    $form['topics'][$obj->nid] = array(
      '#type' => 'markup', 
      '#value' => implode(', ', $node_topics),
    );

    $form['created'][$obj->nid] = array(
      '#type' => 'markup', 
      '#value' => format_date($obj->created, 'custom', 'n/j/y g:i a'),
    );

    // Recent Activity column
    $trail = array();
    
    // Where the item has appeared in the last seven days
    $schedule = db_query("SELECT td.name, td.tid, h.timestamp FROM {slot_machine_history} h LEFT OUTER JOIN {term_data} td ON h.topic = td.tid WHERE nid = %d AND h.timestamp > %d ORDER BY h.timestamp ASC", $obj->nid, time() - (7 * 24 * 60 * 60));
    while ($item = db_fetch_object($schedule)) {
      $trail[] = theme('slot_machine_recent_activity_item', ($item->name ? $item->name : 'Home'), format_date($item->timestamp, 'custom', 'n/j/y'), slot_machine_check_rot($obj->nid, $type, ($item->tid ? $item->tid : 0)));
    }
        
    // Where the item is currently scheduled to appear
    $schedule = db_query("SELECT td.name, td.tid, s.schedule FROM {slot_machine} s LEFT OUTER JOIN {term_data} td ON s.topic = td.tid WHERE nid = %d AND schedule IS NOT NULL AND s.priority > %d ORDER BY schedule ASC", $obj->nid, $max_priority);
    while ($item = db_fetch_object($schedule)) {
      $trail[] = theme('slot_machine_recent_activity_item', ($item->name ? $item->name : 'Home'), format_date($item->schedule, 'custom', 'n/j/y'), slot_machine_check_rot($obj->nid, $type, ($item->tid ? $item->tid : 0)), TRUE);
    }
    $form['recent_activity'][$obj->nid] = array(
      '#type' => 'markup', 
      '#value' => theme('slot_machine_audit_trail', $trail),
    );

    $form['schedule'][$obj->nid] = array(
      '#type' => 'markup', 
      '#value' => ($obj->schedule ? format_date($obj->schedule, 'custom', 'n/j/y g:i a') : 'Never'),
    );
  }

  $form['topic'] = array(
    '#type' => 'value',
    '#value' => $topic,
  );
  $form['type'] = array(
    '#type' => 'value',
    '#value' => $type,
  );
  $form['reset_queue'] = array(
    '#type' => 'button',
    '#value' => t('Reset'),
  );  
  $form['reorder'] = array(
    '#type' => 'submit',
    '#value' => t('Reorder'),
  );
  $form['confirm'] = array(
    '#type' => 'submit',
    '#value' => t('Yes'),
    '#prefix' => theme('slot_machine_reorder_confirmation_popup_prefix'),
    '#suffix' => theme('slot_machine_reorder_confirmation_popup_suffix'),
  );  
  return $form;
}

/*
 * Priority boxes must contain positive integers
 */
function slot_machine_queue_form_validate($form, &$form_state) {
  $error = false;
  foreach ($form_state['values'] as $key => $value) {
    if (substr($key, 0, 9) == 'priority_') {
      if (is_numeric($value) && ($value < 1 || $value - floor($value) != 0)) {
        form_set_error($key, ($error ? ' ' : t('Updated priorities must take the form of a positive integer. (Non-numeric values will be ignored.)')));
        $error = true;
      }
    }
  }
}

/*
 * Update queue ordering and remove selected items.
 */
function slot_machine_queue_form_submit($form, &$form_state) {
  $i = 1;
  $new_order = $priority = array();
  
  // Store the priority fields, excluding those that have been removed
  foreach ($form_state['values'] as $key => $value) {
    $nid = substr($key, 9);
    if (substr($key, 0, 9) == 'priority_' && $form_state['values']['remove_'. $nid] != 1) {
      $new_order[$i++] = '';
      $priority[$form_state['values']['original_priority_'. $nid]] = array(
        'val' => (is_numeric($value) ? $value : $form_state['values']['original_priority_'. $nid]),
        'nid' => $nid,
      );
    }
  }
  krsort($priority);

  // Assign the priority to those items that were explicitly changed
  $error = false;  
  foreach ($priority as $orig => $value) {
    if ($orig != $value['val'] && isset($new_order[$value['val']]) && $new_order[$value['val']] == '') {
      $new_order[$value['val']] = $value;
      $priority[$orig]['val'] = -1;
    }
    elseif ($orig != $value['val']) {
      $priority[$orig]['val'] = $orig;
      if (!$error && $new_order[$value['val']] != '') {
        drupal_set_message(t('ALERT: When you try to give several items the same priority in the queue, only the last item you put in that position will be given that priority ranking. Please make sure that each item you reposition is given a different number.'));
        $error = true;
      }
    }
  }
  ksort($priority);
  
  // Fill in the rest
  foreach ($new_order as $key => $value) {
    if ($value == '') {
      $pvalue = array_shift($priority);
      while ($pvalue['val'] == -1) {
        $pvalue = array_shift($priority);
      }
      $new_order[$key] = $pvalue;
    }
  }

  // Delete and reinsert the queue with new priorities
  $max_priority = db_result(db_query("SELECT priority FROM {slot_machine_slots} WHERE feature_type = '%s' ORDER BY priority DESC LIMIT 1", $form_state['values']['type']));
  db_query("DELETE FROM {slot_machine} WHERE topic = %d AND feature_type = '%s' AND priority > %d", $form_state['values']['topic'], $form_state['values']['type'], $max_priority);
  foreach ($new_order as $key => $value) {
    db_query("INSERT INTO {slot_machine} (nid, topic, feature_type, priority, schedule) VALUES (%d, %d, '%s', %d, NULL)", $value['nid'], $form_state['values']['topic'], $form_state['values']['type'], $key + $max_priority);
  }
}

/*
 * Goes through each item slotted in the slot machine (that isn't published on the current 
 * schedule) and updates the schedule field -- i.e., the exact time each is scheduled to appear, 
 * based on its position in the queue, and that queue's assigned frequency.
 */
function slot_machine_update_slot_schedules() {
  // Get the max priorities
  $max_priorities = array();
  $results = db_query("SELECT * FROM {slot_machine_slots} ORDER BY priority ASC");
  while ($slot = db_fetch_object($results)) {
    $max_priorities[$slot->feature_type] = $slot->priority;
  }

  // Get all topics
  if (variable_get('slot_machine_home_topic', 1)) {
    $topics = array(0);
  } else {
    $topics = array();
  }
  $vocab = variable_get('slot_machine_topics', 'none');
  if ($vocab != 'none') {
    $get_topics = _sm_get_vocabulary_terms($vocab);
    foreach ($get_topics as $key => $mytopic) {
      $topics[] = $key;
    }
  }
  
  // Get all distinct node types (as we only have one queue per node type)
  $feature_types = db_query('SELECT DISTINCT(feature_type) FROM {slot_machine_slots}');
  $types = array();
  while ($row = db_fetch_object($feature_types)) {
    $types[] = $row->feature_type;
  }

  // Now go through each topic and type queue and update the schedule
  $rot_enabled_types = variable_get('slot_machine_enable_content_rotation', array());
  foreach ($topics as $topic) {
    foreach ($types as $type) {
      $max = $max_priorities[$type];
      $qlen = db_result(db_query("SELECT COUNT(1) FROM {slot_machine} WHERE topic = %d AND feature_type = '%s' AND priority > %d", $topic, $type, $max));
      if ($rot_enabled_types[$type]) {
        $rotbase_freq = db_result(db_query('SELECT f.fid FROM {slot_machine_frequencies} f JOIN {slot_machine_slots_frequencies} sf ON f.fid = sf.fid JOIN {slot_machine_slots} s ON sf.sid = s.sid WHERE sf.topic = %d AND s.priority = 0 AND s.feature_type = "%s"', $topic, $type));
      }
      
      // Find all the relevant base schedules
      $base_schedules = array();
      $results = db_query("SELECT f.frequency, f.start, s.priority FROM {slot_machine_slots_frequencies} sf JOIN {slot_machine_slots} s ON sf.sid = s.sid JOIN {slot_machine_frequencies} f ON sf.fid = f.fid WHERE sf.topic = %d AND s.feature_type = '%s' AND f.frequency != 1", $topic, $type);
      while ($obj = db_fetch_object($results)) {
        // We don't need the base schedules for slots that are rotation destinations
        if (!($rot_enabled_types[$type] && $rotbase_freq != 1 && $obj->priority != 0)) {
          $base_schedule = db_result(db_query("SELECT schedule FROM {slot_machine} WHERE topic = %d AND feature_type = '%s' AND priority = %d", $topic, $type, $obj->priority));
          if ($base_schedule && ($obj->frequency || $obj->start)) {
            $base_schedules[] = array('schedule' => $base_schedule, 'frequency' => $obj->frequency, 'start' => $obj->start);
          }
        }
      }
      
      $schedules = array();
      foreach ($base_schedules as $base_schedule) {
        $base = $base_schedule['schedule'];
        if ($base_schedule['start']) {
          $month = format_date($base, 'custom', 'n');
          $day = format_date($base, 'custom', 'j');
          $year = format_date($base, 'custom', 'Y');
          $start = split(':', $base_schedule['start']);
          $start_schedule = mktime($start[0], $start[1], $start[2], $month, $day, $year);
          if ($start_schedule > $base) {
            $base = $start_schedule - (24 * 60 * 60); 
          }
        }
        for ($i = 0; $i < $qlen; $i++) {
          if ($base_schedule['start']) {
            $base += (24 * 60 * 60);
          }
          else {
            $base += $base_schedule['frequency'];
          }
          $schedules[] = $base;
        }
      }      
      sort($schedules);
      
      // Iterate through each item in the queue and shifts in the next scheduled time
      for ($i = $max + 1; $i < $max + $qlen + 1; $i++) {
        db_query("UPDATE {slot_machine} SET schedule = %d WHERE topic = %d AND feature_type = '%s' AND priority = %d", array_shift($schedules), $topic, $type, $i);
      }
    }
  }
}
